'
' OMW 8.1x Integration script by AlarmPoint Systems, Inc.
' All rights reserved
' Version 1, March 2007
' Version 2, September 2007
' Version 2.2, January 2009
' Version 3.0  xMatters, Inc.
'
'===================================================================================================================
' This script updates OMW 8.10 messages via WMI, taking responses from AlarmPoint to update the message as indicated below.
'
' Script Usage:
'	cscript OmwApt  -MsgId <MsgId> -Request <RequestType> [-arg# <Argument#>]
'
' Parameters:
'	-MsgId: <MsgId>
'		is the GUID identifying the Message normally delivered by the VPW variable <$WBEM:TargetInstance.Id>
'		example: 6623fed0-0ff7-926e-1dd4-0f776d4f0000
'
'	-Request:  <RequestType> [-arg1 <Argument1>] [-arg2 <Argument2>]
'		is the Type of Request, with optional arguments as defined below:
'		ApOmwOwn:	Engineer takes ownership of event via AlarmPoint
'				-arg1 <owner_string>
'				-arg2 <optional annotation_string>
'				-omwuser <omw_user>
'				-omwpassword <omw_users_password>
'		ApOmwAck:	Engineer acknowledges event via AlarmPoint
'				-arg1 <optional annotation_string>
'				-omwuser <omw_user>
'				-omwpassword <omw_users_password>
'		ApOmwChgSev:	Engineer changes severity via AlarmPoint
'				-arg1 <severity_string>
'				-arg2 <optional annotation_string>
'				-omwuser <omw_user>
'				-omwpassword <omw_users_password>
'		ApOmwAnnotate:	Engineer annotates event via AlarmPoint	
'				-arg1 <annotation_string>
'				-omwuser <omw_user>
'				-omwpassword <omw_users_password>

Option Explicit

'===================================================================================================================
'=====  Global Variable Declarations
'===================================================================================================================

' Debug Parameters
' When calling the WriteLog Method the logType should be specified,
' "toFile" - Log text to Log File
' "toEvent" - Log text as annotate on original OM-W Event
' "toBoth" - Log text to File and annotate original OM-W Event
' Anything else disables logging behaviour.
' --------------------------------------------------------------------------------------------
' debugLogLevel  - default is false; shows critical logging only - excludes INFO logging
' (true, false)  - when set to true, shows all messages, including INFO logging
'                - note that INFO messages contain a lot of data and can cause large log files 
Dim logType, logText, debugLogLevel
logType = "toFile"
debugLogLevel = false

' Log File Size in bytes (5MB)
Const maxFileSize = 5242880
	
Dim g_objOVMessage		' The OV_Message Object retrieved from Operations Manager via WMI
Set g_objOVMessage = Nothing

Dim messageGuid, g_strRequestType, g_strArg1, g_strArg2, omwuser, omwpassword

Dim omwHost, objSWbemLocator, objSWbemServices, colSwbemObjectSet, objOVMessage

' owmHost should be set to the host which is running HP Operations Manager Windows
omwHost = "localhost"

'===================================================================================================================
Sub Usage ()

' Log/Annotate/Echo the Usage Statement.
'===================================================================================================================
  logText = "Script Usage:" & vbCrLf & vbCrLf _
          & "OmwApt.vbs -MsgId <MsgId> -Request <RequestType> [-arg# <Argument#>]" & vbCrLf & vbCrLf _
          &	"Parameters:" & vbCrLf _
          &	"  -MsgId: <MsgId> is the GUID identifying the Message" & vbCrLf _
          &	"          normally delivered by the VPW variable <$WBEM:TargetInstance.Id>" & vbCrLf _
          &	"  -Request:  <RequestType> [-arg# <Argument#>]" & vbCrLf _
          &	"	   the type of response request, with optional arguments as defined below:" & vbCrLf _
          &	"	ApOmwAck:	Engineer acknowledges event via AlarmPoint." & vbCrLf _
          &	"			-arg1 <optional annotation_string>" & vbCrLf _
          &	"	ApOmwOwn:	Engineer takes ownership of event via AlarmPoint." & vbCrLf _
          &	"			-arg1 <owner_string>" & vbCrLf _
          &	"			-arg2 <optional annotation_string>" & vbCrLf _
          &	"	ApOmwChgSev:	Engineer changes severity" & vbCrLf _
          &	"			-arg1 <severity_string>" & vbCrLf _
          &	"			-arg2 <optional annotation_string>" & vbCrLf _
          &	"	ApOmwAnnotate:	Engineer annotates event via AlarmPoint	." & vbCrLf _	
          &	"			-arg1 <annotation_string>" & vbCrLf & vbCrLf
  Wscript.Echo logText
  logType = "toBoth"
  WriteLog
  WScript.Quit( 1 )  
'===================================================================================================================
End Sub			'Usage		Log/Annotate/Echo the Usage Statement.
'===================================================================================================================

'===================================================================================================================
Sub WriteLog()

' Determine Logging Behaviour
'===================================================================================================================
  On Error Resume Next

  If (debugLogLevel = true OR (debugLogLevel = false And (Left(logText,8) <> "[ INFO ]") ) ) Then
      Select Case logType
        
        Case "toFile"
          LogtoFile
        
        Case "toEvent"
          AnnotateLog

        Case "toBoth"
          LogtoFile
          AnnotateLog

        Case Else
          'Logging disabled
        
      End Select
  End If
'===================================================================================================================
End Sub			'WriteLog		Determine Logging Behaviour
'===================================================================================================================

'===================================================================================================================
Sub LogtoFile()

' Write log entry to file.
'===================================================================================================================
	
	  On Error Resume Next

  Dim strDirectory, strFile, strRollFile
  Dim objFSO, fileSys
  Dim sFile,  objTextFile

  ' Log file roll number  
  Dim logNumber
  Dim logCreated
  
  ' OpenTextFile Method needs a Const value
  ' ForAppending = 8 ForReading = 1, ForWriting = 2
  Const ForAppending = 8
  Const OverwriteExisting = True

  strDirectory = "log"
  strFile = strDirectory & "\XM-OMW-Response.log"
  strRollFile = strDirectory & "\XM-OMW-Response.log."

  ' Create the File System Objects
  Set objFSO = CreateObject("Scripting.FileSystemObject")
  Set fileSys = CreateObject("Scripting.FileSystemObject")

  ' Check that the strDirectory folder exists
  If Not objFSO.FolderExists(strDirectory) Then
    objFSO.CreateFolder(strDirectory)
  End If

  If Not objFSO.FileExists(strFile) Then
    objFSO.CreateTextFile(strFile)
  End If

  Set sFile = fileSys.GetFile(strFile)

  ' Roll Log if exceeding Max File Size
  If sFile.Size > maxFileSize Then
    logNumber = 0
    logCreated = False

    ' Format the log name to include the current date (mm_dd_yyyy_hh_mm_ss_AM/PM)
    Dim currentDate, currentTime, dateString
    currentDate = FormatDateTime(Date(), 2)
    currentTime = FormatDateTime(Now(), 3)
    dateString = Replace(currentDate,"/","_")
    dateString = dateString & "_" & Replace(currentTime,":","_")
    dateString = Replace(dateString," ","_")

    strRollFile = strRollFile & dateString
    
    objFSO.CopyFile strFile, strRollFile, True

    ' Delete log file and recreate it.
    objFSO.DeleteFile(strFile)
    objFSO.CreateTextFile(strFile)
  End If

  Set objTextFile = objFSO.OpenTextFile (strFile, ForAppending, True)

  objTextFile.WriteLine("[ " & date() & " " & time() & " ] " & logText)
  objTextFile.Close

'===================================================================================================================
End Sub			'LogtoFile	 Write log entry to file.
'===================================================================================================================

'===================================================================================================================
Sub AnnotateLog()

' Annotate log message to originating Event.
'===================================================================================================================
  On Error Resume Next

  Dim AnnotationMessage
  AnnotationMessage = logText & " on " & date() & " at " & time()

  If Not g_objOVMessage = Nothing Then
    ' Annotate the Event in OMW with the error message
    HPWMIMessageObject.AddAnnotation( AnnotationMessage )
  Else
  
  logText = "[ INFO ] Attempt to Annotate Log Message Failed, OV_Message Object not retreived." _
	          & "  Original Message: " & AnnotationMessage
	logType = "toFile"
	WriteLog
  End If
'===================================================================================================================
End Sub			'AnnotateLog	Annotate log message to originating Event.
'===================================================================================================================

'===================================================================================================================
Sub ParseCommandLine()
   
' Get Command Line Parameters
'===================================================================================================================
	Dim objCmdArgs, intCmdArgCount, intCmdArgMaxIndex, intCmdArgTypeIndex, intCmdArgValIndex
	Set objCmdArgs = WScript.Arguments
	intCmdArgCount = objCmdArgs.Count
	intCmdArgMaxIndex = intCmdArgCount - 1
		
	Dim strCmdArgType, blnCmdArgsValid
	blnCmdArgsValid=vbTrue
	messageGuid=""
	g_strRequestType=""

	For intCmdArgTypeIndex = 0 To intCmdArgMaxIndex Step 2
		intCmdArgValIndex = intCmdArgTypeIndex + 1
		If ( intCmdArgValIndex <= intCmdArgMaxIndex ) Then
			strCmdArgType = Trim(LCase(objCmdArgs.Item(intCmdArgTypeIndex)))
    
			Select Case strCmdArgType     
        		              	
      			Case "-msgid"
      				messageGuid = Trim(objCmdArgs.Item(intCmdArgValIndex))
					logText = "[ INFO ] Retrieved parameter associated with -MsgId: " & messageGuid
					logType = "toFile"
					WriteLog

				Case "-request"
        			g_strRequestType = Trim(LCase(objCmdArgs.Item(intCmdArgValIndex))) 
					logText = "[ INFO ] Retrieved parameter associated with -Request: " & g_strRequestType
					logType = "toFile"
					WriteLog	
					
				Case "-arg1"
        			g_strArg1 = Trim(objCmdArgs.Item(intCmdArgValIndex))
					logText = "[ INFO ] Retrieved parameter associated with -arg1: " & g_strArg1
					logType = "toFile"
					WriteLog

				Case "-arg2"
        			g_strArg2 = Trim(objCmdArgs.Item(intCmdArgValIndex))
					logText = "[ INFO ] Retrieved parameter associated with -arg2: " & g_strArg2
					logType = "toFile"
					WriteLog

				Case "-omwuser"
        			omwuser = Trim(objCmdArgs.Item(intCmdArgValIndex))
					logText = "[ INFO ] Retrieved parameter associated with -omwuser: " & omwuser
					logType = "toFile"
					WriteLog

				Case "-omwpassword"
        			omwpassword = Trim(objCmdArgs.Item(intCmdArgValIndex))
					logText = "[ INFO ] Retrieved parameter associated with -omwpassword: " & omwpassword
					logType = "toFile"
					WriteLog

				Case Else
      				blnCmdArgsValid = vbFalse

       		End Select

		Else
       		blnCmdArgsValid = vbFalse
       	End If
	Next
	
	If   ( messageGuid = "" Or g_strRequestType = "" ) Then
		logText = "[ FATAL ] Required Parameters Missing for MsgId and/or Type."
		logType = "toFile"
		WriteLog	    
		blnCmdArgsValid = vbFalse
	End If
	
	If ( blnCmdArgsValid = vbFalse) Then
        Usage
		WScript.Quit -1
	End If
	
	Set objCmdArgs = Nothing
'===================================================================================================================
End Sub			'ParseCommandLine		Get Command Line Parameters
'===================================================================================================================

'===================================================================================================================
Sub GetOVMessageObject()

' Get HP WMI OV_Message Object Based on GUID
'===================================================================================================================
  logText = "[ DEBUG ] Getting OV_Message Object: " & messageGuid & vbCrLf
  logType = "toFile"
  WriteLog

  ' Settings for authentication for WMI Calls
  Dim objSWbemLocator, objSWbemServices, colSwbemObjectSet, objOVMessage

  logText = "[ DEBUG ] WMI Authentication: URL: " & omwHost & " USER: " & omwuser & " PASSWORD:" & omwpassword
  logType = "toFile"
  WriteLog

  Set objSWbemLocator = CreateObject("WbemScripting.SWbemLocator")
  Set objSWbemServices = objSWbemLocator.ConnectServer(omwHost, "root\HewlettPackard\OpenView\Data", omwuser, omwpassword)

  Set colSwbemObjectSet = objSWbemServices.ExecQuery("SELECT * FROM OV_MESSAGE WHERE ID = '" & messageGuid & "' ")
  
  For Each objOVMessage in colSWbemObjectSet
    Set g_objOVMessage = objOVMessage
  Next

	If ( Err.Number) Then
		logText = "[ ERROR ] Unable to find OV_Message Object: " & messageGuid & vbCrLf _
			& "Error Number: " & Err.Number & vbCrLf _
			& "No AlarmPoint Message was sent."
		logType = "toFile"
		WriteLog
		WScript.Quit -1
	End If
  
	On Error goto 0
'===================================================================================================================
End Sub			'GetOVMessageObject		Get HP WMI OV_Message Object Based on GUID
'===================================================================================================================

'===================================================================================================================
Sub AcknowledgeOmwMessage()
   
' Send Message to OMW to Acknowledge this Event
'===================================================================================================================
	logText = "[ ACTION ] Acknowledging message " & messageGuid
	logType = "toFile"
	WriteLog
  
  Dim oOutParams
	
  Set oOutParams = g_objOVMessage.ExecMethod_( "Acknowledge")

  If (oOutParams.ReturnValue < 0) Then
    ' Update the Annotation message to include the error
    g_strArg1 = "[ AlarmPoint ] An error occurred while trying to process an Acknowledge action: OV_Message Class WMI Acknowledge() had a ReturnValue: " & oOutParams.ReturnValue
		AddOmwAnnotation()

    logText = "[ ERROR ] Acknowledge OV_Message WMI Call ReturnValue: " & oOutParams.ReturnValue
  Else
    logText = "[ DEBUG ] Acknowledge OV_Message WMI Call ReturnValue: " & oOutParams.ReturnValue
  End If

  logType = "toFile"
  WriteLog

	' Check if optional Annotation value is present
	If ( g_strArg1 <> "" )  Then
		AddOmwAnnotation()
	End If
'===================================================================================================================
End Sub			'AcknowledgeOmwMessage		Send Message to AlarmPoint to Acknowledge this Event
'===================================================================================================================

'===================================================================================================================
Sub OwnOmwMessage()
   
' Send Message to OMW to Own this Event
'===================================================================================================================
	logText = "[ ACTION ] Owning message " & messageGuid
	logType = "toFile"
	WriteLog
	
  Dim oInParams, oOutParams
	
  Set oInParams = g_objOVMessage.Methods_("Own").InParameters.SpawnInstance_
	oInParams.NewOwner = omwuser
	
	Set oOutParams = g_objOVMessage.ExecMethod_( "Own", oInParams)

  If (oOutParams.ReturnValue < 0) Then
    ' Update the Annotation message to include the error
    g_strArg1 = "[ AlarmPoint ] An error occurred while trying to process an Own action: OV_Message Class WMI Own() had a ReturnValue: " & oOutParams.ReturnValue
		AddOmwAnnotation()

    logText = "[ ERROR ] Own OV_Message WMI Call ReturnValue: " & oOutParams.ReturnValue
  Else
    logText = "[ DEBUG ] Own OV_Message WMI Call ReturnValue: " & oOutParams.ReturnValue
  End If
  logType = "toFile"
  WriteLog

	' Check if optional Annotation value is present
	If ( g_strArg2 <> "" )  Then
		' AddOmwAnnotation expects the annotation in the global g_strArg1, 
		' but this call has it in g_strArg2, so we have to re-assign it.
		g_strArg1 = g_strArg2
		AddOmwAnnotation()
	End If
'===================================================================================================================
End Sub			'OwnOmwMessage		Send Message to AlarmPoint to Own this Event
'===================================================================================================================

'===================================================================================================================
Sub ChangeOmwSeverity()
   
' Change the Severity of the Event Id
'===================================================================================================================
	logText = "[ ACTION ] Changing severity of message " & messageGuid & " to " & g_strArg1
	logType = "toFile"
	WriteLog

  Dim oInParams, oOutParams
  Set oInParams = g_objOVMessage.Methods_("ChangeSeverity").InParameters.SpawnInstance_

	Select Case (LCase(g_strArg1))
  		Case "normal"  	oInParams.NewSeverity = "2"
  		Case "warning"  oInParams.NewSeverity = "4"
  		Case "minor"  	oInParams.NewSeverity = "8"
  		Case "major" 	  oInParams.NewSeverity = "16"      
  		Case "critical" oInParams.NewSeverity = "32"
	End Select

	Set oOutParams = g_objOVMessage.ExecMethod_( "ChangeSeverity", oInParams)
  
  If (oOutParams.ReturnValue < 0) Then
    ' Update the Annotation message to include the error
    g_strArg1 = "[ AlarmPoint ] An error occurred while trying to process an ChangeSeverity action: OV_Message Class WMI ChangeSeverity() had a ReturnValue: " & oOutParams.ReturnValue
		AddOmwAnnotation()

    logText = "[ ERROR ] ChangeSeverity OV_Message WMI Call ReturnValue: " & oOutParams.ReturnValue
  Else
    logText = "[ DEBUG ] ChangeSeverity OV_Message WMI Call ReturnValue: " & oOutParams.ReturnValue
  End If
  logType = "toFile"
  WriteLog

	' Check if optional Annotation value is present
	If ( g_strArg2 <> "" )  Then
		' AddOmwAnnotation expects the annotation in the global g_strArg1, 
		' but this call has it in g_strArg2, so we have to re-assign it.
		g_strArg1 = g_strArg2
		AddOmwAnnotation()
	End If
'===================================================================================================================
End Sub			'ChangeOmwSeverity	Change the severity of the Event Id
'===================================================================================================================

'===================================================================================================================
Sub AddOmwAnnotation()
   
' Add an Annotation to this Event Id
'===================================================================================================================
	logText = "[ ACTION ] Adding annotation """ & g_strArg1 & """ to message " & messageGuid
	logType = "toFile"
	WriteLog

  Dim oInParams, oOutParams
  Set oInParams = g_objOVMessage.Methods_("AddAnnotation").InParameters.SpawnInstance_
	oInParams.Text = g_strArg1
	
	Set oOutParams = g_objOVMessage.ExecMethod_( "AddAnnotation", oInParams)

  logText = "[ DEBUG ] AddAnnotation OV_Message WMI Call ReturnValue: " & oOutParams.ReturnValue
  logType = "toFile"
  WriteLog

  'Wscript.Echo "Annotation Text: " & oOutParams.Item("Text")

'===================================================================================================================
End Sub			'AddOmwAnnotation	Add an Annotation to this Event Id
'===================================================================================================================

'===================================================================================================================
'===== Main Program
'===================================================================================================================
ParseCommandLine
GetOVMessageObject

logText = "[ DEBUG ] g_strRequestType = " & g_strRequestType
logType = "toBoth"
WriteLog

Select Case g_strRequestType
	Case "apomwown"
		OwnOmwMessage()

	Case "apomwack"
		AcknowledgeOmwMessage()
		
	Case "apomwchgsev"
		ChangeOmwSeverity()

	Case "apomwannotate"
		AddOmwAnnotation()

	Case Else
		logText = "[ ERROR ] Unknown Omw RequestType from AlarmPoint" & g_strRequestType & VbCrLf
		logType = "toBoth"
		WriteLog
		WScript.Quit -1
End Select
WScript.Quit 0
